@@ -1,7 +1,7 @@
 /*
   gsl.c
   Ruby/GSL: Ruby extension library for GSL (GNU Scientific Library)
-    (C) Copyright 2001-2004 by Yoshiki Tsunesada
+    (C) Copyright 2001-2006 by Yoshiki Tsunesada
 
   Ruby/GSL is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License.
@@ -88,7 +88,7 @@ void Init_rb_gsl()
   Init_gsl_eigen(mgsl);
 
   Init_gsl_fft(mgsl);
-  /*  Init_gsl_signal(mgsl);*/
+    Init_gsl_signal(mgsl);
   Init_gsl_function(mgsl);
   Init_gsl_integration(mgsl);
 
@@ -146,6 +146,28 @@ void Init_rb_gsl()
   Init_gsl_graph(mgsl);
   Init_gsl_dirac(mgsl);
 
+#ifdef HAVE_TAMU_ANOVA_TAMU_ANOVA_H
+  Init_tamu_anova(mgsl);
+#endif
+
+#ifdef HAVE_OOL_OOL_VERSION_H
+	Init_ool(mgsl);
+#endif
+
+#ifdef HAVE_JACOBI_H
+	Init_jacobi(mgsl);
+#endif
+
+#ifdef HAVE_GSL_GSL_CQP_H
+	Init_cqp(mgsl);
+#endif
+
+	Init_fresnel(mgsl);
+	
+#ifdef GSL_1_9_LATER
+	Init_bspline(mgsl);
+#endif
+
   rb_gsl_define_methods(mgsl);
 }
 
