@@ -1,7 +1,7 @@
 /*
   interp.c
   Ruby/GSL: Ruby extension library for GSL (GNU Scientific Library)
-    (C) Copyright 2001-2004 by Yoshiki Tsunesada
+    (C) Copyright 2001-2006 by Yoshiki Tsunesada
 
   Ruby/GSL is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License.
@@ -10,11 +10,10 @@
 */
 
 #include "rb_gsl_config.h"
-
 #include "rb_gsl_interp.h"
 
 VALUE cgsl_interp_accel; /* this is used also in spline.c */
-EXTERN VALUE cgsl_vector;
+EXTERN VALUE cgsl_vector, cgsl_matrix;
 
 static void rb_gsl_interp_free(rb_gsl_interp *sp);
 
@@ -189,7 +188,13 @@ static VALUE rb_gsl_interp_evaluate(VALUE obj, VALUE xxa, VALUE yya, VALUE xx,
 #endif
   Data_Get_Struct(obj, rb_gsl_interp, rgi);
   ptrx = get_vector_ptr(xxa, &stridex, &size);
+  if (size != rgi->p->size ){
+    rb_raise(rb_eTypeError, "size mismatch (xa:%d != %d)", size, rgi->p->size);
+  }
   ptry = get_vector_ptr(yya, &stridey, &size);
+  if (size != rgi->p->size ){
+    rb_raise(rb_eTypeError, "size mismatch (ya:%d != %d)", size, rgi->p->size);
+  }
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
   case T_FIXNUM:  case T_BIGNUM:  case T_FLOAT:
@@ -214,7 +219,7 @@ static VALUE rb_gsl_interp_evaluate(VALUE obj, VALUE xxa, VALUE yya, VALUE xx,
       ptrz = (double*) na->ptr;
       ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
       ptr = NA_PTR_TYPE(ary, double*);
-      for (i = 0; i < size; i++)
+      for (i = 0; i < na->total; i++)
 	ptr[i] = (*eval)(rgi->p, ptrx, ptry, ptrz[i], rgi->a);
       return ary;
     }
@@ -465,7 +470,6 @@ void Init_gsl_interp(VALUE module)
   cgsl_interp = rb_define_class_under(module, "Interp", cGSL_Object);
   cgsl_interp_accel = rb_define_class_under(cgsl_interp, "Accel", cGSL_Object);
 
-  rb_define_singleton_method(cgsl_interp, "new", rb_gsl_interp_new, -1);
   rb_define_singleton_method(cgsl_interp, "alloc", rb_gsl_interp_new, -1);
 
   rb_gsl_interp_define_const(cgsl_interp);
