@@ -13,6 +13,11 @@
 #include "rb_gsl_function.h"
 #include "rb_gsl_common.h"
 
+#ifdef HAVE_NDLINEAR_GSL_MULTIFIT_NDLINEAR_H
+#include <ndlinear/gsl_multifit_ndlinear.h>
+void Init_ndlinear(VALUE module);
+#endif
+
 #ifndef CHECK_WORKSPACE
 #define CHECK_WORKSPACE(x) if(CLASS_OF(x)!=cgsl_multifit_workspace)\
       rb_raise(rb_eTypeError,\
@@ -40,7 +45,7 @@ static VALUE rb_gsl_multifit_workspace_new(VALUE klass, VALUE n, VALUE p)
   gsl_multifit_linear_workspace *w = NULL;
   CHECK_FIXNUM(n); CHECK_FIXNUM(p);
   w = gsl_multifit_linear_alloc(FIX2INT(n), FIX2INT(p));
-  return Data_Wrap_Struct(klass, 0, gsl_multifit_linear_free, w);
+  return Data_Wrap_Struct(cgsl_multifit_workspace, 0, gsl_multifit_linear_free, w);
 }
 
 static VALUE rb_gsl_multifit_linear(int argc, VALUE *argv, VALUE obj)
@@ -560,6 +565,8 @@ static VALUE rb_gsl_multifit_function_fdf_set_procs(int argc, VALUE *argv, VALUE
   rb_ary_store(ary, 0, argv[0]);
   rb_ary_store(ary, 1, argv[1]);
   switch (argc) {
+	case 2:
+		break;
   case 3:
     if (TYPE(argv[2]) == T_FIXNUM) {
       func->p = FIX2INT(argv[2]);
@@ -586,7 +593,6 @@ static VALUE rb_gsl_multifit_function_fdf_set_data(int argc, VALUE *argv, VALUE
 {
   VALUE ary, ary2;
   gsl_multifit_function_fdf *func = NULL;
-  gsl_vector *v = NULL;
   Data_Get_Struct(obj, gsl_multifit_function_fdf, func);
   if (func->params == NULL) {
     ary = rb_ary_new2(4);
@@ -606,8 +612,7 @@ static VALUE rb_gsl_multifit_function_fdf_set_data(int argc, VALUE *argv, VALUE
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
     break;
   }
-  Data_Get_Vector(argv[0], v);
-  func->n = v->size;
+  func->n = NUM2INT(rb_funcall(argv[0], rb_intern("size"), 0));
   rb_ary_store(ary, 3, ary2);
   return obj;
 }
@@ -625,13 +630,13 @@ static int gsl_multifit_function_fdf_f(const gsl_vector *x, void *params,
   case 2:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
-    rb_funcall(proc, ID_call, 4, vx, vt, vy, vf);
+    rb_funcall(proc, RBGSL_ID_call, 4, vx, vt, vy, vf);
     break;
   case 3:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
     vsigma = rb_ary_entry(vt_y_sigma,2);
-    rb_funcall(proc, ID_call, 5, vx, vt, vy, vsigma, vf);
+    rb_funcall(proc, RBGSL_ID_call, 5, vx, vt, vy, vsigma, vf);
     break;
   default:
     rb_raise(rb_eArgError, "bad argument");
@@ -653,13 +658,13 @@ static int gsl_multifit_function_fdf_df(const gsl_vector *x, void *params,
   case 2:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
-    rb_funcall(proc, ID_call, 4, vx, vt, vy, vJ);
+    rb_funcall(proc, RBGSL_ID_call, 4, vx, vt, vy, vJ);
     break;
   case 3:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
     vsigma = rb_ary_entry(vt_y_sigma,2);
-    rb_funcall(proc, ID_call, 5, vx, vt, vy, vsigma, vJ);
+    rb_funcall(proc, RBGSL_ID_call, 5, vx, vt, vy, vsigma, vJ);
     break;
   default:
     rb_raise(rb_eArgError, "bad argument");
@@ -686,20 +691,20 @@ static int gsl_multifit_function_fdf_fdf(const gsl_vector *x, void *params,
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
     if (NIL_P(proc_fdf)) {
-      rb_funcall(proc_f, ID_call, 4, vx, vt, vy, vf);
-      rb_funcall(proc_df, ID_call, 4, vx, vt, vy, vJ);
+      rb_funcall(proc_f, RBGSL_ID_call, 4, vx, vt, vy, vf);
+      rb_funcall(proc_df, RBGSL_ID_call, 4, vx, vt, vy, vJ);
     } else 
-      rb_funcall(proc_fdf, ID_call, 5, vx, vt, vy, vf, vJ);
+      rb_funcall(proc_fdf, RBGSL_ID_call, 5, vx, vt, vy, vf, vJ);
     break;
   case 3:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
     vsigma = rb_ary_entry(vt_y_sigma,2);
     if (NIL_P(proc_fdf)) {
-      rb_funcall(proc_f, ID_call, 5, vx, vt, vy, vsigma, vf);
-      rb_funcall(proc_df, ID_call, 5, vx, vt, vy, vsigma, vJ);
+      rb_funcall(proc_f, RBGSL_ID_call, 5, vx, vt, vy, vsigma, vf);
+      rb_funcall(proc_df, RBGSL_ID_call, 5, vx, vt, vy, vsigma, vJ);
     } else 
-      rb_funcall(proc_fdf, ID_call, 6, vx, vt, vy, vsigma, vf, vJ);
+      rb_funcall(proc_fdf, RBGSL_ID_call, 6, vx, vt, vy, vsigma, vf, vJ);
     break;
   default:
     rb_raise(rb_eArgError, "bad argument");
@@ -1647,6 +1652,22 @@ static VALUE rb_gsl_multifit_fit(int argc, VALUE *argv, VALUE klass)
 		     rb_float_new(chi2), INT2FIX(dof));
 }
 
+#ifdef GSL_1_8_LATER
+static VALUE rb_gsl_multifit_linear_est(VALUE module, VALUE xx, VALUE cc, VALUE ccov)
+{
+  gsl_vector *x, *c;
+  gsl_matrix *cov;
+  double y, y_err;
+
+  Data_Get_Vector(xx, x);
+  Data_Get_Vector(cc, c);
+  Data_Get_Matrix(ccov, cov);
+  gsl_multifit_linear_est(x, c, cov, &y, &y_err);
+  return rb_ary_new3(2, rb_float_new(y), rb_float_new(y_err));
+}
+
+#endif
+
 void Init_gsl_multifit(VALUE module)
 {
   VALUE mgsl_multifit;
@@ -1661,6 +1682,8 @@ void Init_gsl_multifit(VALUE module)
 			     rb_gsl_multifit_workspace_new, 2);
   rb_define_singleton_method(cgsl_multifit_workspace, "alloc", 
 			     rb_gsl_multifit_workspace_new, 2);
+  rb_define_singleton_method(mgsl_multifit, "alloc", 
+			     rb_gsl_multifit_workspace_new, 2);			     
   rb_define_module_function(mgsl_multifit, "linear", rb_gsl_multifit_linear, -1);
   rb_define_module_function(mgsl_multifit, "wlinear", rb_gsl_multifit_wlinear, -1);
 
@@ -1734,6 +1757,16 @@ void Init_gsl_multifit(VALUE module)
 
   /*****/
   rb_define_singleton_method(cgsl_multifit_fdfsolver, "fit", rb_gsl_multifit_fit, -1);
+
+  /***/
+#ifdef GSL_1_8_LATER
+  rb_define_module_function(mgsl_multifit, "linear_est", rb_gsl_multifit_linear_est, 3);
+#endif
+
+#ifdef HAVE_NDLINEAR_GSL_MULTIFIT_NDLINEAR_H
+  Init_ndlinear(mgsl_multifit);
+#endif
+
 }
 
 #ifdef CHECK_WORKSPACE
