@@ -1,7 +1,7 @@
 /*
   vector.c
   Ruby/GSL: Ruby extension library for GSL (GNU Scientific Library)
-    (C) Copyright 2001-2004 by Yoshiki Tsunesada
+    (C) Copyright 2001-2006 by Yoshiki Tsunesada
         Modified by Seiya Nishizawa        14/Apr/2004
 
   Ruby/GSL is free software: you can redistribute it and/or modify it
@@ -541,11 +541,11 @@ static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
       draw_narray(argv[0], fp);
 #endif
     } else {
-      if (fp) fclose(fp);
+      if (fp) pclose(fp);
       rb_raise(rb_eTypeError, "wrong argument type %s", 
 	       rb_class2name(CLASS_OF(argv[0])));
     }
-    if (fp) fclose(fp);
+    if (fp) pclose(fp);
     return Qtrue;
   } else {
     fp = popen(command, "w");
@@ -593,7 +593,7 @@ static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
       for (j = 0; j < n; j++) gsl_vector_set(x, j, (double) j);
       flag = 1;
     } else {
-      if (fp) fclose(fp);
+      if (fp) pclose(fp);
       rb_raise(rb_eTypeError, "wrong argument type %s", 
 	       rb_class2name(CLASS_OF(argv[0])));
     }
@@ -834,7 +834,7 @@ static VALUE rb_gsl_vector_plot2(int argc, VALUE *argv, VALUE obj)
   }
   fprintf(fp, "e\n");
   fflush(fp);
-  fclose(fp);
+  pclose(fp);
   fp = NULL;
   return Qtrue;
 }
@@ -852,7 +852,8 @@ static VALUE rb_gsl_vector_sqrt(VALUE obj)
 static VALUE rb_gsl_vector_normalize(int argc, VALUE *argv, VALUE obj)
 {
   gsl_vector *v = NULL, *vnew = NULL;
-  double dnrm2, nrm;
+  double mean;
+  double sd, nrm;
   switch (argc) {
   case 0:
     nrm = 1.0;
@@ -867,15 +868,18 @@ static VALUE rb_gsl_vector_normalize(int argc, VALUE *argv, VALUE obj)
   }
   Data_Get_Vector(obj, v);
   vnew = make_vector_clone(v);
-  dnrm2 = gsl_blas_dnrm2(vnew);
-  gsl_vector_scale(vnew, sqrt(nrm)/dnrm2);
+  mean = gsl_stats_mean(v->data, v->stride, v->size);
+  gsl_vector_add_constant(vnew, -mean);
+  sd = gsl_stats_sd(vnew->data, vnew->stride, vnew->size);  
+  gsl_vector_scale(vnew, sqrt(nrm)/sd);
   return Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, vnew);  
 }
 
 static VALUE rb_gsl_vector_normalize_bang(int argc, VALUE *argv, VALUE obj)
 {
   gsl_vector *v = NULL;
-  double dnrm2, nrm;
+  double mean;  
+  double sd, nrm;
   switch (argc) {
   case 0:
     nrm = 1.0;
@@ -889,8 +893,10 @@ static VALUE rb_gsl_vector_normalize_bang(int argc, VALUE *argv, VALUE obj)
     break;
   }
   Data_Get_Vector(obj, v);
-  dnrm2 = gsl_blas_dnrm2(v);
-  gsl_vector_scale(v, sqrt(nrm)/dnrm2);
+  mean = gsl_stats_mean(v->data, v->stride, v->size);
+  gsl_vector_add_constant(v, -mean);  
+  sd = gsl_stats_sd(v->data, v->stride, v->size);  
+  gsl_vector_scale(v, sqrt(nrm)/sd);
   return obj;
 }
 
@@ -910,7 +916,7 @@ static VALUE rb_gsl_vector_filescan_na(VALUE klass, VALUE file)
   sprintf(buf, "wc %s", filename);
   fp = popen(buf, "r");
   fgets(buf, 1024, fp);
-  fclose(fp);
+  pclose(fp);
   sscanf(buf, "%d", &nn);
   lines = (size_t) nn;      /* vector length */
   shape[0] = lines;
@@ -1043,6 +1049,294 @@ static VALUE rb_gsl_vector_log10(VALUE obj)
   return rb_gsl_sf_eval1(log10, obj);
 }
 
+static VALUE rb_gsl_vector_rotate_bang(int argc, VALUE *argv, VALUE klass)
+{
+  double rad;
+  double x, y, c, s;
+  gsl_vector *vx, *vy;
+  VALUE v0, v1, retval;
+  size_t i, n;
+  switch (argc) {
+  case 2:
+    if (TYPE(argv[0]) == T_ARRAY) {
+      v0 = rb_ary_entry(argv[0], 0);
+      v1 = rb_ary_entry(argv[0], 1);
+      if (VECTOR_P(v0) && VECTOR_P(v1)) {
+	Data_Get_Struct(v0, gsl_vector, vx);
+	Data_Get_Struct(v1, gsl_vector, vy);
+	n = (size_t) GSL_MIN(vx->size, vy->size);
+	rad = NUM2DBL(argv[1]);
+	retval = argv[0];
+      } else {
+	x = NUM2DBL(rb_ary_entry(argv[0], 0));
+	y = NUM2DBL(rb_ary_entry(argv[0], 1));
+	rad = NUM2DBL(argv[1]);
+	c = cos(rad); s = sin(rad);
+	return rb_ary_new3(2, rb_float_new(c*x - s*y), rb_float_new(s*x + c*y));
+      }
+    } else {
+      rb_raise(rb_eTypeError, "wrong argument type %s (Array expected)",
+	       rb_class2name(CLASS_OF(argv[0])));
+    }
+    break;
+  case 3:
+    if (VECTOR_P(argv[0]) && VECTOR_P(argv[1])) {
+      Data_Get_Struct(argv[0], gsl_vector, vx);
+      Data_Get_Struct(argv[1], gsl_vector, vy);
+      n = (size_t) GSL_MIN(vx->size, vy->size);
+      rad = NUM2DBL(argv[1]);
+      retval = rb_ary_new3(2, argv[0], argv[1]);
+    } else {
+      x = NUM2DBL(argv[0]);
+      y = NUM2DBL(argv[1]);
+      rad = NUM2DBL(argv[2]);
+      c = cos(rad); s = sin(rad);
+      return rb_ary_new3(2, rb_float_new(c*x - s*y), rb_float_new(s*x + c*y));
+    }
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
+    break;
+  }
+  c = cos(rad); s = sin(rad);
+  for (i = 0; i < n; i++) {
+    x = gsl_vector_get(vx, i);
+    y = gsl_vector_get(vy, i);
+    gsl_vector_set(vx, i, c*x - s*y);
+    gsl_vector_set(vy, i, s*x + c*y);
+  }
+  return retval;
+}
+
+static VALUE rb_gsl_vector_rotate(int argc, VALUE *argv, VALUE klass)
+{
+  double rad;
+  double x, y, c, s;
+  gsl_vector *vx, *vy, *vxnew, *vynew;
+  VALUE v0, v1;
+  size_t i, n;
+  switch (argc) {
+  case 2:
+    if (TYPE(argv[0]) == T_ARRAY) {
+      v0 = rb_ary_entry(argv[0], 0);
+      v1 = rb_ary_entry(argv[0], 1);
+      if (VECTOR_P(v0) && VECTOR_P(v1)) {
+	Data_Get_Struct(v0, gsl_vector, vx);
+	Data_Get_Struct(v1, gsl_vector, vy);
+	rad = NUM2DBL(argv[1]);
+      } else {
+	x = NUM2DBL(rb_ary_entry(argv[0], 0));
+	y = NUM2DBL(rb_ary_entry(argv[0], 1));
+	rad = NUM2DBL(argv[1]);
+	c = cos(rad); s = sin(rad);
+	return rb_ary_new3(2, rb_float_new(c*x - s*y), rb_float_new(s*x + c*y));
+      }
+    } else {
+      rb_raise(rb_eTypeError, "wrong argument type %s (Array expected)",
+	       rb_class2name(CLASS_OF(argv[0])));
+    }
+    break;
+  case 3:
+    if (VECTOR_P(argv[0]) && VECTOR_P(argv[1])) {
+      Data_Get_Struct(argv[0], gsl_vector, vx);
+      Data_Get_Struct(argv[1], gsl_vector, vy);
+      rad = NUM2DBL(argv[1]);
+    } else {
+      x = NUM2DBL(argv[0]);
+      y = NUM2DBL(argv[1]);
+      rad = NUM2DBL(argv[2]);
+      c = cos(rad); s = sin(rad);
+      return rb_ary_new3(2, rb_float_new(c*x - s*y), rb_float_new(s*x + c*y));
+    }
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
+    break;
+  }
+  n = (size_t) GSL_MIN(vx->size, vy->size);
+  vxnew = gsl_vector_alloc(n);
+  vynew = gsl_vector_alloc(n);
+  c = cos(rad); s = sin(rad);
+  for (i = 0; i < n; i++) {
+    x = gsl_vector_get(vx, i);
+    y = gsl_vector_get(vy, i);
+    gsl_vector_set(vxnew, i, c*x - s*y);
+    gsl_vector_set(vynew, i, s*x + c*y);
+  }
+  return rb_ary_new3(2, Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vxnew),
+		     Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vynew));
+}
+
+#include "gsl/gsl_fit.h"
+
+static VALUE rb_gsl_vector_linearfit(int argc, VALUE *argv, VALUE klass)
+{
+  gsl_vector *x, *y, *w = NULL;
+  double c0, c1, c00, c01, c11, sumsq;
+  switch (argc) {
+  case 3:
+    CHECK_VECTOR(argv[0]); CHECK_VECTOR(argv[1]); CHECK_VECTOR(argv[2]);
+    Data_Get_Struct(argv[0], gsl_vector, x);
+    Data_Get_Struct(argv[1], gsl_vector, w);
+    Data_Get_Struct(argv[2], gsl_vector, y);
+    gsl_fit_wlinear(x->data, x->stride, w->data, w->stride,
+		    y->data, y->stride, y->size, &c0, &c1, &c00, &c01, &c11,
+		    &sumsq);
+    break;
+  case 2:
+    CHECK_VECTOR(argv[0]); CHECK_VECTOR(argv[1]); 
+    Data_Get_Struct(argv[0], gsl_vector, x);
+    Data_Get_Struct(argv[1], gsl_vector, y);
+    gsl_fit_linear(x->data, x->stride, y->data,y->stride, y->size, 
+		   &c0, &c1, &c00, &c01, &c11, &sumsq);
+    break;
+  default:
+    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 2 or 3).\n", argc);
+  }
+  return rb_ary_new3(6, rb_float_new(c0), rb_float_new(c1), rb_float_new(c00),
+		    rb_float_new(c01), rb_float_new(c11), rb_float_new(sumsq));
+}
+
+static VALUE rb_gsl_vector_center(VALUE obj)
+{
+  gsl_vector *v, *vnew;
+  double mean;
+  Data_Get_Struct(obj, gsl_vector, v);
+  mean = gsl_stats_mean(v->data, v->stride, v->size);
+  vnew = gsl_vector_alloc(v->size);
+  gsl_vector_memcpy(vnew, v);
+  gsl_vector_add_constant(vnew, -mean);
+  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+}
+
+static VALUE rb_gsl_vector_clip(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector *v, *vnew;  
+  double hi = 1.0, lo = 0.0;
+  double x;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  switch (argc) {
+  case 0:
+    break;
+  case 1:
+    if (TYPE(argv[0]) == T_ARRAY) {
+      lo = NUM2DBL(rb_ary_entry(argv[0], 0));
+      hi = NUM2DBL(rb_ary_entry(argv[0], 1));
+    } else {
+      hi = NUM2DBL(argv[0]);
+    }
+    break;
+  case 2:
+    lo = NUM2DBL(argv[0]);
+    hi = NUM2DBL(argv[1]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-2).\n", argc);
+  }
+  vnew = gsl_vector_alloc(v->size);
+  for (i = 0; i < v->size; i++) {
+    x = gsl_vector_get(v, i);
+    if (x > hi) x = hi;
+    else if (x < lo) x = lo;
+    else;
+    gsl_vector_set(vnew, i, x);
+  }
+  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+}
+
+static VALUE rb_gsl_vector_amp_phase(VALUE obj)
+{
+  gsl_vector *v;
+  gsl_vector *amp, *phase;
+  double re, im;
+  VALUE vamp, vphase;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  amp = gsl_vector_alloc(v->size/2);
+  phase = gsl_vector_alloc(v->size/2);
+  gsl_vector_set(amp, 0, gsl_vector_get(v, 0));
+  gsl_vector_set(phase, 0, 0);  
+  gsl_vector_set(amp, amp->size-1, gsl_vector_get(v, v->size-1));
+  gsl_vector_set(phase, phase->size-1, 0);    
+  for (i = 1; i < v->size-1; i+=2) {
+    re = gsl_vector_get(v, i);
+    im = gsl_vector_get(v, i+1);    
+    gsl_vector_set(amp, i/2+1, sqrt(re*re + im*im));
+    gsl_vector_set(phase, i/2+1, atan2(im, re));
+  }
+  vamp = Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, amp);
+  vphase = Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, phase);
+  return rb_ary_new3(2, vamp, vphase);
+}
+
+
+static VALUE rb_gsl_vector_clean(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector *v = NULL, *vnew = NULL;
+  double eps = 1e-10;
+  size_t n, i;
+  switch (argc) {
+  case 0:
+    /* do nothing */
+    break;
+  case 1:
+    Need_Float(argv[0]);
+    eps = NUM2DBL(argv[0]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "too many arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  Data_Get_Struct(obj, gsl_vector, v);
+  vnew = make_vector_clone(v);
+  n = v->size;
+  for (i = 0; i < n; i++) if (fabs(vnew->data[i]) < eps) vnew->data[i] = 0.0;
+  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);;
+}
+
+static VALUE rb_gsl_vector_clean_bang(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector *v = NULL;
+  double eps = 1e-10;
+  size_t n, i;
+  switch (argc) {
+  case 0:
+    /* do nothing */
+    break;
+  case 1:
+    Need_Float(argv[0]);
+    eps = NUM2DBL(argv[0]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "too many arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  Data_Get_Struct(obj, gsl_vector, v);
+  n = v->size;
+  for (i = 0; i < n; i++) if (fabs(v->data[i]) < eps) v->data[i] = 0.0;
+  return obj;
+}
+
+VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn);
+VALUE rb_gsl_vector_pow(VALUE obj, VALUE p)
+{
+  return rb_gsl_pow(Qnil, obj, p);
+}
+
+VALUE rb_gsl_vector_pow_bang(VALUE obj, VALUE pp)
+{
+  gsl_vector *v;
+  double p;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  p = NUM2DBL(pp);
+  for (i = 0; i < v->size; i++) {
+    gsl_vector_set(v, i, pow(gsl_vector_get(v, i), p));
+  }
+  return obj;
+}
+
 void Init_gsl_vector(VALUE module)
 {
   rb_define_singleton_method(cgsl_vector, "linspace", rb_gsl_vector_linspace, -1);
@@ -1114,5 +1408,22 @@ void Init_gsl_vector(VALUE module)
   rb_define_method(cgsl_vector, "log", rb_gsl_vector_log, 0);
   rb_define_method(cgsl_vector, "log10", rb_gsl_vector_log10, 0);
 
+  rb_define_singleton_method(cgsl_vector, "rotate", rb_gsl_vector_rotate, -1);
+  rb_define_singleton_method(cgsl_vector, "rotate!", rb_gsl_vector_rotate_bang, -1);
+
+  rb_define_singleton_method(cgsl_vector, "linearfit", rb_gsl_vector_linearfit, -1);
+
+  rb_define_method(cgsl_vector, "center", rb_gsl_vector_center, 0);
+  rb_define_method(cgsl_vector, "clip", rb_gsl_vector_clip, -1);
+
+  rb_define_method(cgsl_vector, "amp_phase", rb_gsl_vector_amp_phase, 0);
+  rb_define_method(cgsl_vector, "clean", rb_gsl_vector_clean, -1);
+  rb_define_method(cgsl_vector, "clean!", rb_gsl_vector_clean_bang, -1);
+  
+  rb_define_method(cgsl_vector, "pow", rb_gsl_vector_pow, 1);
+  rb_define_alias(cgsl_vector, "**", "pow");
+  
+  rb_define_method(cgsl_vector, "pow!", rb_gsl_vector_pow_bang, 1);  
+  
   Init_gsl_vector_init(module);
 }
