@@ -2,7 +2,7 @@
 #define ___RB_GSL_CONFIG_H___
 
 #ifndef GSL_VERSION
-#define GSL_VERSION "1.6"
+#define GSL_VERSION "1.10"
 #endif
 #ifndef GSL_0_9_4_LATER
 #define GSL_0_9_4_LATER
@@ -31,8 +31,20 @@
 #ifndef GSL_1_6_LATER
 #define GSL_1_6_LATER
 #endif
+#ifndef GSL_1_8_LATER
+#define GSL_1_8_LATER
+#endif
+#ifndef GSL_1_9_LATER
+#define GSL_1_9_LATER
+#endif
+#ifndef GSL_1_10_LATER
+#define GSL_1_10_LATER
+#endif
+#ifndef HAVE_EIGEN_FRANCIS
+#define HAVE_EIGEN_FRANCIS
+#endif
 #ifndef RUBY_GSL_VERSION
-#define RUBY_GSL_VERSION "1.6.3"
+#define RUBY_GSL_VERSION "1.10.3"
 #endif
 #ifndef RUBY_1_8_LATER
 #define RUBY_1_8_LATER
